#include "mainwindow.h"
# include <math.h>
class RastriginProblem:public Problem
{
    public:
        RastriginProblem(): Problem(2)
        {

            Data x1,x2;
            x1.resize(2);
            x2.resize(2);
            x1[0]=-1;x1[1]=-1;
            x2[0]= 1;x2[1]= 1;
            setLeftMargin(x1);
            setRightMargin(x2);
        }
        virtual double funmin(Data &x)
        {
            return x[0]*x[0]+x[1]*x[1]-cos(18.0*x[0])-cos(18.0*x[1]);
        }
};

const int defaultGenerations=100;
const int defaultParticles=100;
RastriginProblem problem;

MainWindow::MainWindow(QWidget *parent) :
    QMainWindow(parent)
{
    setFixedSize(500,500);
    setWindowTitle("Plot Demo");
    mainWidget=new QWidget;
    setCentralWidget(mainWidget);
    mainWidget->resize(this->width(),this->height());
    mainLayout=new QVBoxLayout;
    mainWidget->setLayout(mainLayout);
    pointSeries=new QScatterSeries;
    chart=new QChart();
    chart->legend()->hide();
    chart->addSeries(pointSeries);
    chart->createDefaultAxes();
    chart->setTitle("PSO");
    chartView = new QChartView(chart);
    chartView->setRenderHint(QPainter::Antialiasing);
    mainLayout->addWidget(chartView);

    QHBoxLayout *line1=new QHBoxLayout;
    mainLayout->addLayout(line1);
    c1Edit=new QLineEdit;
    c1Edit->setPlaceholderText("c1 value");
    c1Edit->setValidator(new QDoubleValidator(0.0,100.0,0.5));
    line1->addWidget(c1Edit);
    c2Edit=new QLineEdit;
    c2Edit->setPlaceholderText("c2 value");
    c2Edit->setValidator(new QDoubleValidator(0.0,100.0,0.5));
    line1->addWidget(c2Edit);
    weightEdit=new QLineEdit;
    weightEdit->setPlaceholderText("Weight value");
    weightEdit->setValidator(new QDoubleValidator(0.0,10.0,0.1));
    line1->addWidget(weightEdit);

    QHBoxLayout *line2=new QHBoxLayout;
    mainLayout->addLayout(line2);
    particleEdit=new QLineEdit;
    particleEdit->setPlaceholderText("Particles");
    particleEdit->setValidator(new QIntValidator(0,1000));
    line2->addWidget(particleEdit);
    generationsEdit=new QLineEdit;
    generationsEdit->setPlaceholderText("Generations");
    generationsEdit->setValidator(new QIntValidator(0,1000));
    line2->addWidget(generationsEdit);
    stepButton=new QPushButton;
    stepButton->setText("STEP");
    line2->addWidget(stepButton);
    connect(stepButton,SIGNAL(clicked(bool)),this,SLOT(stepslot()));
    setButton=new QPushButton;
    setButton->setText("Set Params");
    connect(setButton,SIGNAL(clicked(bool)),this,SLOT(setSlot()));
    line2->addWidget(setButton);

    QHBoxLayout *line3=new QHBoxLayout;
    mainLayout->addLayout(line3);
    generationLabel=new QLabel;
    generationLabel->setText("GENERATION: ");
    line3->addWidget(generationLabel);
    bestLabel=new QLabel;
    bestLabel->setText("BEST VALUE: ");
    line3->addWidget(bestLabel);
    currentGeneration=0;

    algorithm=new Pso(defaultParticles,&problem);
}

void MainWindow::setSlot()
{
    algorithm->setweight(weightEdit->text().toDouble());
    algorithm->setc1(c1Edit->text().toDouble());
    algorithm->setc2(c2Edit->text().toDouble());
}

void    MainWindow::stepslot()
{
    algorithm->nextGeneration();
    ++currentGeneration;
    generationLabel->setText("GENERATION: "+QString::number(currentGeneration));
    bestLabel->setText("BEST VALUE: "+QString::number(algorithm->getBestFitness()));
    pointSeries->append(currentGeneration,algorithm->getBestFitness());
    chart->removeSeries(pointSeries);
    chart->addSeries(pointSeries);
    chart->createDefaultAxes();
    chartView->repaint();
}

MainWindow::~MainWindow()
{

}
